@@ -40,3 +40,9 @@
 
 /* Define if strptime is present AND works */
 #undef HAVE_STRPTIME
+
+@BOTTOM@
+
+/* Define for backwards compatibility */
+#define ZFT_OBSOLETE
+
