@@ -2,7 +2,7 @@
 #define _ZFTAPE_H
 
 /*
- * Copyright (C) 1996, 1997 Claus-Justus Heine.
+ * Copyright (C) 1996-1998 Claus-Justus Heine.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 
  *
  * $RCSfile: zftape.h,v $
- * $Revision: 1.2 $
- * $Date: 1998/12/18 22:45:53 $
+ * $Revision: 1.3 $
+ * $Date: 1999/08/31 10:07:44 $
  *
  *      Special ioctl and other global info for the zftape VFS
  *      interface for the QIC-40/80/3010/3020 floppy-tape driver for
@@ -35,20 +35,38 @@
 #define ZFTAPE_LABEL       "Ftape - The Linux Floppy Tape Project!"
 
 /* Bits of the minor device number that control the operation mode */
-#define ZFT_Q80_MODE		(1 << 3)
-#define ZFT_RAW_MODE		(1 << 5)
-#define ZFT_MINOR_OP_MASK	(ZFT_Q80_MODE	| 	\
-				 ZFT_RAW_MODE)
-#define ZFT_MINOR_MASK		(FTAPE_SEL_MASK		|	\
-				 ZFT_MINOR_OP_MASK	|	\
-				 FTAPE_NO_REWIND)
 
-#ifdef ZFT_OBSOLETE
+#ifndef ZFT_OBSOLETE
+# define ZFT_MINOR_OP_MASK 0
+#else
+
+/* NOTE: all of these bits are considered OBSOLETE. In-kernel
+ * compression isn't supported any longer, and the "raw mode" stuff is
+ * controlled using an ioctl
+ */
+# define ZFT_Q80_MODE		(1 << 3) 
+# define ZFT_RAW_MODE		(1 << 5)
+# if defined(CONFIG_ZFT_COMPRESSOR) || defined(CONFIG_ZFT_COMPRESSOR_MODULE)
+/* read-only compatibility with pre-ftape-4.x compressed volumes */
+#  define ZFT_ZIP_MODE		(1 << 4)
+#  define ZFT_MINOR_OP_MASK	(ZFT_Q80_MODE	| 	\
+				 ZFT_ZIP_MODE	| 	\
+				 ZFT_RAW_MODE)
+# else
+#  define ZFT_MINOR_OP_MASK	(ZFT_Q80_MODE	| 	\
+				 ZFT_RAW_MODE)
+# endif /* CONFIG_ZFT_COMPRESSOR || CONFIG_ZFT_COMPRESSOR_MODULE */
+
 struct mtblksz {
 	unsigned int mt_blksz;
 };
-#define MTIOC_ZFTAPE_GETBLKSZ _IOR('m', 104, struct mtblksz)
-#endif
+# define MTIOC_ZFTAPE_GETBLKSZ _IOR('m', 104, struct mtblksz)
+#endif /* ZFT_OBSOLETE */
+
+/* valid bits of minor device number */
+#define ZFT_MINOR_MASK		(FTAPE_SEL_MASK		|	\
+				 ZFT_MINOR_OP_MASK	|	\
+				 FTAPE_NO_REWIND)
 
 #ifdef __KERNEL__
 
