@@ -17,8 +17,8 @@
 
  *
  * $RCSfile: ftfmt-tapelib.c,v $
- * $Revision: 1.10 $
- * $Date: 1999/03/17 11:25:42 $
+ * $Revision: 1.12 $
+ * $Date: 2000/06/25 21:14:27 $
  *
  *      This program contains the tape manipulation functions for the
  *      user level floppy tape formatting stuff for the
@@ -49,6 +49,7 @@
 #include <linux/ftape.h>
 #include <linux/ftape-vendors.h>
 #include <linux/ftape-header-segment.h>
+#include <linux/zftape.h>
 
 #include "ftformat.h"
 #include "ftfmt-options.h"
@@ -96,6 +97,52 @@ void *tape_mmap(int tape_fd, size_t dma_size)
     return dma_buffer;
 }
 
+#ifndef MTIOCFTMODE
+#define raw_mode_onoff(tape_fd, verbose, onoff) 0
+#else
+static int raw_mode_onoff(int tape_fd, int verbose, int onoff)
+{
+    struct stat statb;
+    struct mtftmode mtftmode;
+    
+    /* determine minor number of this ftape device
+     */
+    if (fstat(tape_fd, &statb) == -1) {
+	perror(_("Error: Can't stat tape device file"));
+	return -1;
+    }
+    /* See whether we have been called with a "raw" ftape device,
+     * otherwise switch the driver to raw mode
+     */
+    if (!S_ISCHR(statb.st_mode)) {
+	/* should not happen any more at this point */
+	fprintf(stderr,_("Error: tape file is not a character device!"));
+	return -1;
+    }
+#ifdef ZFT_OBSOLETE
+    if (minor(statb.st_rdev) & ZFT_RAW_MODE) {
+	if (verbose) {
+	    printf(_("Using old ftape raw device with minor number 0x%02x\n"),
+		   minor(statb.st_rdev));
+	}
+    } else
+#endif	
+    {
+	/* try switching to raw mode using ioctl */
+	mtftmode.ft_rawmode = onoff ? 1 : 0;
+	if (ioctl(tape_fd, MTIOCFTMODE, &mtftmode) == -1) {
+	    perror(_("Error switching to raw mode"));
+	    return -1;
+	}
+	if (verbose) {
+	    printf(_("Switched %s raw mode using MTIOCFTRAW ioctl\n"),
+		   onoff ? "to" : "off");
+	}
+    }
+    return 0;
+}
+#endif
+
 /*
  *  open the tape device, try to determine whether it is a floppy tape,
  *  and store the hardware status in drive_config, resp. tape_config.
@@ -123,6 +170,9 @@ int tape_reopen_rw(const char *name, int tape_fd)
 	perror(_("Error re-opening tape dev read/write"));
 	return -1;
     }
+    /* no need to call raw_mode_onoff() again, the driver remembers its
+     * state.
+     */
     return tape_fd;
 }
 
@@ -263,6 +313,14 @@ int tape_open(const char *name,
     tpparms->drive_config   = hw_status.status.drive_config;
     tpparms->tape_status    = hw_status.status.tape_status;
     tpparms->vendor         = vendor;
+    /* now switch to raw mode using ioctl or check for presence of raw
+     * mode bit in minor device number
+     */
+    if (raw_mode_onoff(tape_fd, opts->verbose, 1) == -1) {
+	(void)close(tape_fd);
+	return -1;
+    }
+
     return tape_fd;
 }
 
@@ -281,6 +339,7 @@ void tape_close(const int tape_fd, void *dma_buffer, const size_t dma_size)
 		perror(_("Error unmapping dma buffer"));
 	    }
 	}
+	(void)raw_mode_onoff(tape_fd, 0, 0);
 	if (close(tape_fd) == -1) {
 	    perror(_("Error closing tape dev"));
 	}
