@@ -22,8 +22,8 @@
  */
 
 char src[] = "$RCSfile: libftvt.c,v $";
-char rev[] = "$Revision: 1.6 $";
-char dat[] = "$Date: 1999/03/04 09:29:03 $";
+char rev[] = "$Revision: 1.10 $";
+char dat[] = "$Date: 2000/06/25 16:05:47 $";
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -167,7 +167,72 @@ const ftvt_ctrl *ftvt_get_ctrl(void)
 	return &ctrl;
 }
 
-/* Open the tape device, try to determine whether is is a floppy tape.
+#ifndef MTIOCFTMODE
+#define raw_mode_onoff(tape_fd, onoff) 0
+#else
+static int raw_mode_onoff(int tape_fd, int onoff)
+{
+	struct stat statb;
+	struct mtftmode mtftmode;
+
+	/* determine minor number of this ftape device
+	 */
+	if (fstat(tape_fd, &statb) == -1) {
+		ftvt_perror(_("Error: Can't stat tape device file"));
+		return -1;
+	}
+	/* See whether we have been called with a "raw" ftape device,
+	 * otherwise switch the driver to raw mode
+	 */
+	if (!S_ISCHR(statb.st_mode)) {
+		/* should not happen any more at this point */
+		ftvt_error(_("Error: tape file is not a character device!"));
+		return -1;
+	}
+	/* first check whether the ioctl is supported. We do this by
+	 * switching to normal mode. If raw mode was intented, we switch
+	 * back later.
+	 */
+	mtftmode.ft_rawmode = 0;
+	if (ioctl(tape_fd, MTIOCFTMODE, &mtftmode) == -1) {
+		/* not supported */
+#ifdef ZFT_OBSOLETE
+		/* Ok, in this case we don't really switch to and from raw
+		 * mode, but simply check for the correct device.
+		 */
+		if (onoff) {
+			if (minor(statb.st_rdev) & ZFT_RAW_MODE) {
+				if (verbose) {
+		printf(_("Using old ftape raw device with minor number 0x%02x\n"),
+				   minor(statb.st_rdev));
+				}
+			} else {
+		ftvt_error(_("Error: raw mode bit not set in minor device number\n"));
+				return -1;
+			}
+		}
+#else
+		ftvt_perror(_("Error switching to normal mode"));
+		return -1;
+#endif
+	} else {
+		if (onoff) {
+			mtftmode.ft_rawmode = 1;
+			if (ioctl(tape_fd, MTIOCFTMODE, &mtftmode) == -1) {
+				ftvt_perror(_("Error switching to raw mode"));
+				return -1;
+			}
+		}
+		if (verbose) {
+			printf(_("Switched %s raw mode using MTIOCFTMODE ioctl\n"),
+				   onoff ? _("to") : _("off"));
+		}
+	}
+	return 0;
+}
+#endif
+
+/* Open the tape device, try to determine whether it is a floppy tape.
  *  
  * Returns the tape fd on success, -1 otherwise
  *
@@ -225,6 +290,11 @@ int ftvt_open(const char *name, mode_t mode)
 		}
 		drivetype = vendor->name;
 	}
+	if (raw_mode_onoff(tape_fd, 1) == -1) {
+		(void)close(tape_fd);
+		return -1;
+	}
+
 	return tape_fd;
 }
 
@@ -239,6 +309,9 @@ int ftvt_close(int tape_fd)
 		ftvt_perror(_("Ioctl error rewinding tape"));
 		result = -1;
 	}
+	if (raw_mode_onoff(tape_fd, 0) == -1) {
+		result = -1;
+	}
 	if (close(tape_fd) == -1) {
 		ftvt_perror(_("Error closing tape device"));
 		result = -1;
@@ -522,6 +595,12 @@ static char *ftvt_gets(FILE *fp, char *buffer, int *len)
 
 	for (;;) {
 		errno = 0;
+		if (remaining == 0) {
+			buffer = xrealloc(buffer, 1024 + *len);
+			p = buffer + *len;
+			(*len) += 1024;
+			remaining = 1024;
+		}
 		switch (c = getc(fp)) {
 		case EOF: {
 			*p = '\0';
@@ -535,16 +614,11 @@ static char *ftvt_gets(FILE *fp, char *buffer, int *len)
 			return buffer;				
 		}
 		default: {
-			if (remaining <= 1) {
-				buffer = xrealloc(buffer, 1024 + *len);
-				p = buffer + *len - remaining;
-				(*len) += 1024;
-				remaining += 1024;
-			}
 			*(p++) = (char)c;
 			break;
 		}
 		}
+		remaining --;
 	}
 }
 
